@@ -33,6 +33,8 @@ class JVCProjector:
         self.PJ_REQ: Final = ACKs.pj_req.value
         self.client = None
         self.command_read_timeout = 3
+        # NZ or NX (NP5 is classified as NX)
+        self.model_family = ""
 
     def open_connection(self) -> bool:
         """Open a connection"""
@@ -96,18 +98,40 @@ class JVCProjector:
             return result, False
 
         # try sending PJREQ, if there's an error, raise exception
-        # TODO: error handling
-        self.client.sendall(pj_req)
-        
-        # see if we receive PJACK, if not, raise exception
-        msg_pjack = self.client.recv(len(self.PJ_ACK))
-        if msg_pjack != self.PJ_ACK:
-            result = f"Exception with PJACK: {msg_pjack}"
-            self.logger.error(result)
-            return result, False
-        self.logger.debug("Handshake successful")
+        try:
+            self.client.sendall(pj_req)
+            
+            # see if we receive PJACK, if not, raise exception
+            msg_pjack = self.client.recv(len(self.PJ_ACK))
+            if msg_pjack != self.PJ_ACK:
+                result = f"Exception with PJACK: {msg_pjack}"
+                self.logger.error(result)
+                return result, False
+            self.logger.debug("Handshake successful")
+        except socket.timeout:
+            return "handshake timeout", False
+        # Get model family
+        self.model_family = self._get_modelfamily()
+        self.logger.debug("Model code is %s", self.model_family)
         return "ok", True
 
+    def _get_modelfamily(self) -> str:
+        cmd = (
+            Header.reference.value
+            + Header.pj_unit.value
+            + Commands.get_model.value
+            + Footer.close.value
+        )
+
+        res, _ = self._send_command(
+            cmd,
+            command_type=Header.reference.value,
+            ack=ACKs.model.value,
+        )
+        nz_models = ["B5A3", "B5A2", "B5A1"]
+        # check if last 4 are in the nz list
+        return  "NZ" if self._replace_headers(res).decode()[-4:] in nz_models else "NX"
+
     def _check_closed(self) -> bool:
         try:
             # this will try to read bytes without blocking and also without removing them from buffer (peek only)
@@ -195,7 +219,7 @@ class JVCProjector:
         command: bytes,
         ack: bytes,
         command_type: bytes = b"!",
-    ) -> tuple[str, bool]:
+    ) -> tuple[Union[str, bytes], bool]:
         retry_count = 0
         if self.client is None:
             self.logger.debug("Forming connection")
@@ -228,10 +252,8 @@ class JVCProjector:
 
             # Receive the acknowledgement from PJ
             try:
-                # seems like certain commands timeout when PJ is off
-                received_ack = self.client.recv(len(ack_value))
-                # second_message = self.client.recv(len(ack_value))
-                # self.logger.debug(f"two ack: {two}")
+                # most commands timeout when PJ is off
+                received_msg = self.client.recv(len(ack_value))
             except socket.timeout:
                 self.logger.error(
                     "Connection timed out. Command %s is probably not allowed to run at this time.",
@@ -245,44 +267,39 @@ class JVCProjector:
                 return
 
             except ConnectionRefusedError:
-                self.logger.error("Connection Refused when getting ack")
+                self.logger.error("Connection Refused when getting msg")
                 self.logger.debug("restarting connection")
                 self.client.close()
                 self.reconnect()
                 retry_count += 1
                 continue
 
-            self.logger.debug("received ack from PJ: %s", received_ack)
+            self.logger.debug("received msg from PJ: %s", received_msg)
 
-            # This will probably never happen since we are handling timeouts now
-            if received_ack == b"":
-                self.logger.error("Got a blank ack. Restarting connection")
+            # This is unlikely to happen unless we read blank response
+            if received_msg == b"":
+                self.logger.error("Got a blank msg. Restarting connection")
                 self.client.close()
                 self.reconnect()
                 retry_count += 1
                 continue
 
             # get the ack for operation
-            if received_ack == ack_value and command_type == Header.operation.value:
-                return received_ack, True
+            if received_msg == ack_value and command_type == Header.operation.value:
+                return received_msg, True
 
             # if we got what we expect and this is a reference,
             # receive the data we requested
-            if received_ack == ack_value and command_type == Header.reference.value:
-                # Need to read +1 because it sends a \n at the end.... probably a code smell
-                message = self.client.recv(len(ack_value)+1)
+            if received_msg == ack_value and command_type == Header.reference.value:
+                message = self.client.recv(1000)
                 self.logger.debug("received message from PJ: %s", message)
 
                 return message, True
 
-            # if second_message == b"\n":
-            #     retry_count += 1
-            #     continue
-            # Otherwise, it failed
             # Because this now reuses a connection, reaching this stage means catastrophic failure, or HA running as usual :)
             self.logger.error(
                 "Received ack did not match expected ack: %s != %s",
-                received_ack,
+                received_msg,
                 ack_value,
             )
             # Try to restart connection, if we got here somethihng is out of sync
